@@ -2,7 +2,7 @@
 //
 // RcppEigenWrap.h: Rcpp wrap methods for Eigen matrices, vectors and arrays
 //
-// Copyright (C)      2011 Douglas Bates, Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2011 - 2012   Douglas Bates, Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of RcppEigen.
 //
@@ -154,87 +154,123 @@ namespace Rcpp{
 
 
     namespace traits {
-
         /* support for Rcpp::as */
-    
+        
+        template <typename T, int RTYPE>
+        class Eigen_Matrix_Exporter {
+            public:
+            Eigen_Matrix_Exporter(SEXP x) : vec(x), d_ncol(1), d_nrow(Rf_length(x)) {
+                if (TYPEOF(x) != RTYPE)
+                    throw std::invalid_argument("Wrong R type for mapped vector");
+                if (::Rf_isMatrix(x)) {
+                    int *dims = vec.dims() ;
+                    d_nrow = dims[0];
+                    d_ncol = dims[1];
+                }
+            }
+            T get() {return T(vec.begin(), d_nrow, d_ncol );}
+            private:
+                Rcpp::Vector<RTYPE> vec ;
+                int d_nrow, d_ncol ;
+        } ;
+
+        // modified from "class MatrixExporter" in <Rcpp>/include/Rcpp/internal/Exporter.h
+        // MatrixExporter uses brackets [] to index the destination matrix,
+        // which is not supported by MatrixXd.
+        // Here we copy data to MatrixXd.data() rather than MatrixXd
+        template <typename T, typename value_type>
+        class MatrixExporterForEigen {
+        public:
+            typedef value_type r_export_type;
+
+            MatrixExporterForEigen(SEXP x) : object(x){}
+            ~MatrixExporterForEigen(){}
+
+            T get() {
+                Shield<SEXP> dims( ::Rf_getAttrib( object, R_DimSymbol ) );
+                if( Rf_isNull(dims) || ::Rf_length(dims) != 2 ){
+                    throw ::Rcpp::not_a_matrix();
+                }
+                int* dims_ = INTEGER(dims);
+                T result( dims_[0], dims_[1] );
+                value_type *data = result.data();
+                ::Rcpp::internal::export_indexing<value_type*, value_type>( object, data );
+                return result ;
+            }
+
+        private:
+            SEXP object;
+        };
+        
+
         template<typename T>
         class Exporter<Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, 1> > > {
+            typedef typename Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, 1> > OUT ;
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Rcpp::Vector<RTYPE> vec ;
+        
         public:
-            typedef typename Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, 1> >  MVType;
-            Exporter(SEXP x) : d_size(::Rf_length(x)) {
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : vec(x) {
                 if (TYPEOF(x) != RTYPE)
                     throw std::invalid_argument("Wrong R type for mapped vector");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(x);
             }
-            MVType get() {return MVType(d_start, d_size);}
-        protected:
-            const int d_size;
-            T*        d_start;
-        };
-
+            OUT get() {return OUT(vec.begin(), vec.size());}
+        } ;
+        
         template<typename T>
-        class Exporter<Eigen::Map<Eigen::Array<T, Eigen::Dynamic, 1> > > {
+        class Exporter< Eigen::Map<Eigen::Array<T, Eigen::Dynamic, 1> > > {
+            typedef typename Eigen::Map<Eigen::Array<T, Eigen::Dynamic, 1> > OUT ;
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Rcpp::Vector<RTYPE> vec ;
+        
         public:
-            typedef typename Eigen::Map<Eigen::Array<T, Eigen::Dynamic, 1> >  MAType;
-            Exporter(SEXP x) : d_size(::Rf_length(x)) {
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : vec(x) {
                 if (TYPEOF(x) != RTYPE)
                     throw std::invalid_argument("Wrong R type for mapped vector");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(x);
             }
-            MAType get() {return MAType(d_start, d_size);}
-        protected:
-            const int d_size;
-            T*        d_start;
-        };
-
+            OUT get() {return OUT(vec.begin(), vec.size());}
+        } ;
+        
         template<typename T>
         class Exporter<Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > > {
-        public:
-            typedef typename Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >  MMType;
-            Exporter(SEXP x) : d_nrow(::Rf_length(x)), d_ncol(1) {
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            typedef typename Eigen::Map<Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> > OUT ;
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Rcpp::Vector<RTYPE> vec ;
+            int d_ncol, d_nrow ;
+            
+            public:
+            Exporter(SEXP x) : vec(x), d_ncol(1), d_nrow(Rf_length(x)) {
                 if (TYPEOF(x) != RTYPE)
                     throw std::invalid_argument("Wrong R type for mapped vector");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(x);
                 if (::Rf_isMatrix(x)) {
-                    int *dims = INTEGER(::Rf_getAttrib(x, R_DimSymbol));
+                    int *dims = INTEGER( ::Rf_getAttrib( x, R_DimSymbol ) ) ;
                     d_nrow = dims[0];
                     d_ncol = dims[1];
                 }
             }
-            MMType get() {return MMType(d_start, d_nrow, d_ncol);}
-        protected:
-            int   d_nrow, d_ncol;
-            T*    d_start;
-        };
-
+            OUT get() {return OUT(vec.begin(), d_nrow, d_ncol );}
+        } ;  
+        
         template<typename T>
         class Exporter<Eigen::Map<Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic> > > {
-        public:
-            typedef typename Eigen::Map<Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic> >  MAType;
-            Exporter(SEXP x) : d_nrow(::Rf_length(x)), d_ncol(1) {
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            typedef typename Eigen::Map<Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic> > OUT ;
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Rcpp::Vector<RTYPE> vec ;
+            int d_ncol, d_nrow ;
+            
+            public:
+            Exporter(SEXP x) : vec(x), d_ncol(1), d_nrow(Rf_length(x)) {
                 if (TYPEOF(x) != RTYPE)
                     throw std::invalid_argument("Wrong R type for mapped vector");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(x);
                 if (::Rf_isMatrix(x)) {
-                    int *dims = INTEGER(::Rf_getAttrib(x, R_DimSymbol));
+                    int *dims = INTEGER( ::Rf_getAttrib( x, R_DimSymbol ) ) ;
                     d_nrow = dims[0];
                     d_ncol = dims[1];
                 }
             }
-            MAType get() {return MAType(d_start, d_nrow, d_ncol);}
-        protected:
-            int   d_nrow, d_ncol;
-            T*    d_start;
-        };
-
+            OUT get() {return OUT(vec.begin(), d_nrow, d_ncol );}
+        } ;  
+        
         template <typename T> 
         class Exporter<Eigen::Matrix<T, Eigen::Dynamic, 1> >
             : public IndexingExporter<Eigen::Matrix<T, Eigen::Dynamic, 1>, T> {
@@ -255,105 +291,113 @@ namespace Rcpp{
         public:
             Exporter(SEXP x) : IndexingExporter< Eigen::Matrix<T, 1, Eigen::Dynamic>, T >(x){}
         }; 
+
+        template <typename T> 
+        class Exporter< Eigen::Array<T, 1, Eigen::Dynamic> >
+            : public IndexingExporter< Eigen::Array<T, 1, Eigen::Dynamic>, T > {
+        public:
+            Exporter(SEXP x) : IndexingExporter< Eigen::Array<T, 1, Eigen::Dynamic>, T >(x){}
+        }; 
         
         template <typename T> 
         class Exporter< Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic> >
-            : public MatrixExporter< Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, T > {
+            : public MatrixExporterForEigen< Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, T > {
         public:
             Exporter(SEXP x) :
-                MatrixExporter< Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, T >(x){}
+                MatrixExporterForEigen< Eigen::Matrix<T, Eigen::Dynamic, Eigen::Dynamic>, T >(x){}
         }; 
 
         template <typename T> 
         class Exporter< Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic> >
-            : public MatrixExporter< Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic>, T > {
+            : public MatrixExporterForEigen< Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic>, T > {
         public:
             Exporter(SEXP x) :
-                MatrixExporter< Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic>, T >(x){}
+                MatrixExporterForEigen< Eigen::Array<T, Eigen::Dynamic, Eigen::Dynamic>, T >(x){}
         }; 
 
         template<typename T>
         class Exporter<Eigen::MappedSparseMatrix<T> > {
         public:
-            Exporter(SEXP x)
-                : d_x(x), d_dims(d_x.slot("Dim")), d_i(d_x.slot("i")), d_p(d_x.slot("p")) {
-                if (!d_x.is("CsparseMatrix")) 
-                    throw std::invalid_argument("Need S4 class CsparseMatrix for an mapped sparse matrix");
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
-                SEXP xx = d_x.slot("x");
-                if (TYPEOF(xx) != RTYPE)
-                    throw std::invalid_argument("Wrong R type for mapped sparse matrix");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(xx);
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : d_x(x), d_dims(d_x.slot("Dim")), d_i(d_x.slot("i")), d_p(d_x.slot("p")), xx( d_x.slot("x") ) {
+                if (!d_x.is("dgCMatrix")) 
+                    throw std::invalid_argument("Need S4 class dgCMatrix for a mapped sparse matrix");
             }
             Eigen::MappedSparseMatrix<T> get() {
                 return Eigen::MappedSparseMatrix<T>(d_dims[0], d_dims[1], d_p[d_dims[1]],
-                                                    d_p.begin(), d_i.begin(), d_start);
+                                                    d_p.begin(), d_i.begin(), xx.begin() );
             }
         protected:
             S4            d_x;
-            T*            d_start;
             IntegerVector d_dims, d_i, d_p;
+            Vector<RTYPE> xx ;
+        };
+
+        template<typename T>
+        class Exporter<Eigen::MappedSparseMatrix<T, Eigen::RowMajor> > {
+        public:
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : d_x(x), d_dims(d_x.slot("Dim")), d_j(d_x.slot("j")), d_p(d_x.slot("p")), xx( d_x.slot("x") ) {
+                if (!d_x.is("dgRMatrix")) 
+                    throw std::invalid_argument("Need S4 class dgRMatrix for a mapped sparse matrix");
+            }
+            Eigen::MappedSparseMatrix<T, Eigen::RowMajor> get() {
+                return Eigen::MappedSparseMatrix<T, Eigen::RowMajor>(d_dims[0], d_dims[1], d_p[d_dims[1]],
+                                                                     d_p.begin(), d_j.begin(), xx.begin() );
+            }
+        protected:
+            S4            d_x;
+            IntegerVector d_dims, d_j, d_p;
+            Vector<RTYPE> xx ;
         };
 
         template<typename T>
         class Exporter<Eigen::SparseMatrix<T> > {
         public:
-            Exporter(SEXP x)
-                : d_x(x), d_dims(d_x.slot("Dim")), d_i(d_x.slot("i")), d_p(d_x.slot("p")) {
-                if (!d_x.is("CsparseMatrix"))
-                    throw std::invalid_argument("Need S4 class CsparseMatrix for an mapped sparse matrix");
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
-                SEXP xx = d_x.slot("x");
-                if (TYPEOF(xx) != RTYPE) // should coerce instead - see Rcpp/inst/include/Rcpp/internal/export.h
-                    throw std::invalid_argument("Wrong R type for sparse matrix");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(xx);
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            Exporter(SEXP x) : d_x(x), d_dims(d_x.slot("Dim")), d_i(d_x.slot("i")), d_p(d_x.slot("p")), xx(d_x.slot("x")) {
+                if (!d_x.is("dgCMatrix"))
+                    throw std::invalid_argument("Need S4 class dgCMatrix for a sparse matrix");
             }
             Eigen::SparseMatrix<T> get() {
                 Eigen::SparseMatrix<T>  ans(d_dims[0], d_dims[1]);
                 ans.reserve(d_p[d_dims[1]]);
                 for(int j = 0; j < d_dims[1]; ++j) {
                     ans.startVec(j);
-                    for (int k = d_p[j]; k < d_p[j + 1]; ++k) ans.insertBack(d_i[k], j) = d_start[k];
+                    for (int k = d_p[j]; k < d_p[j + 1]; ++k) ans.insertBack(d_i[k], j) = xx[k];
                 }
                 ans.finalize();  
                 return ans;
             }
         protected:
             S4            d_x;
-            T*            d_start;
             IntegerVector d_dims, d_i, d_p;
+            Vector<RTYPE> xx ;
         };
         
         template<typename T>
         class Exporter<Eigen::SparseMatrix<T, Eigen::RowMajor> > {
         public:
-            Exporter(SEXP x)
-                : d_x(x), d_dims(d_x.slot("Dim")), d_j(d_x.slot("j")), d_p(d_x.slot("p")) {
+            const static int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
+            
+            Exporter(SEXP x) : d_x(x), d_dims(d_x.slot("Dim")), d_j(d_x.slot("j")), d_p(d_x.slot("p")), xx(d_x.slot("x")) {
                 if (!d_x.is("dgRMatrix"))
                     throw std::invalid_argument("Need S4 class dgRMatrix for a sparse matrix");
-                const int RTYPE = ::Rcpp::traits::r_sexptype_traits<T>::rtype ;
-                SEXP xx = d_x.slot("x");
-                if (TYPEOF(xx) != RTYPE) // should coerce instead - see Rcpp/inst/include/Rcpp/internal/export.h
-                    throw std::invalid_argument("Wrong R type for sparse matrix");
-                typedef typename ::Rcpp::traits::storage_type<RTYPE>::type STORAGE;
-                d_start         = ::Rcpp::internal::r_vector_start<RTYPE,STORAGE>(xx);
             }
             Eigen::SparseMatrix<T, Eigen::RowMajor> get() {
                 Eigen::SparseMatrix<T, Eigen::RowMajor>  ans(d_dims[0], d_dims[1]);
                 ans.reserve(d_p[d_dims[0]]);
                 for(int i = 0; i < d_dims[0]; ++i) {
                     ans.startVec(i);
-                    for (int k = d_p[i]; k < d_p[i + 1]; ++k) ans.insertBack(i, d_j[k]) = d_start[k];
+                    for (int k = d_p[i]; k < d_p[i + 1]; ++k) ans.insertBack(i, d_j[k]) = xx[k];
                 }
                 ans.finalize();  
                 return ans;
             }
         protected:
             S4            d_x;
-            T*            d_start;
             IntegerVector d_dims, d_j, d_p;
+            Vector<RTYPE> xx ;
         };
 
     } // namespace traits
